@@ -1,4 +1,3 @@
-mod aggregated; pub use aggregated::*;
 mod cell; pub use cell::*;
 mod context; pub use context::*;
 mod dict; pub use dict::*;
@@ -8,7 +7,7 @@ mod time; pub use time::*;
 #[cfg(test)] mod test;
 
 pub trait Mergable: Sized {
-	type Diff;
+	type Diff: Diff;
 
 	fn merge(&mut self, other: Self) {
 		self.apply(Mergable::diff(&other, self))
@@ -24,6 +23,10 @@ pub trait Mergable: Sized {
 	fn apply(&mut self, diff: Self::Diff) -> Result<(), ApplyError>;
 }
 
+pub trait Diff: Sized {
+	fn revert(self) -> Result<Self, RevertError>;
+}
+
 #[derive(Debug,thiserror::Error)]
 #[non_exhaustive]
 pub enum ApplyError {
@@ -31,5 +34,14 @@ pub enum ApplyError {
 	Missing(String),
 }
 
+#[derive(Debug,thiserror::Error)]
+#[non_exhaustive]
+pub enum RevertError {
+	#[error("Cannot revert, value has been undone and redone too many times.")]
+	TooManyRedos,
+	#[error("Apply target missing critical information: {0:?}")]
+	Mismatch(String),
+}
+
 #[derive(Clone,Debug)]
 pub struct Opaque<T>(T);
